rsh: Syntactic Sugar for Rush Manager Construction

View source: R/sugar.R

rshR Documentation

Syntactic Sugar for Rush Manager Construction

Description

Function to construct a Rush manager.

Usage

rsh(network_id = NULL, config = NULL, ...)

Arguments

network_id

(character(1))
Identifier of the rush network. Manager and workers must have the same id. Keys in Redis are prefixed with the instance id.

config

(redux::redis_config)
Redis configuration options. If NULL, configuration set by rush_plan() is used. If rush_plan() has not been called, the REDIS_URL environment variable is parsed. If REDIS_URL is not set, a default configuration is used. See redux::redis_config for details.

...

(ignored).

Value

Rush manager.

Examples

if (redux::redis_available()) {
   config_local = redux::redis_config()
   rush = rsh(network_id = "test_network", config = config_local)
   rush
}

rush documentation built on March 18, 2026, 5:08 p.m.